==> kuzu-0.0.9.tar.gz: sdist/kuzu-source/src/storage/store/var_list_column_chunk.cpp <==
#include "storage/store/var_list_column_chunk.h"

#include "arrow/array.h"
#include "common/types/value/nested.h"
#include "common/types/value/value.h"
#include "storage/store/table_copy_utils.h"

using namespace kuzu::common;

namespace kuzu {
namespace storage {

void VarListDataColumnChunk::reset() {
    dataColumnChunk->resetToEmpty();
}

void VarListDataColumnChunk::resizeBuffer(uint64_t numValues) {
    if (numValues <= capacity) {
        return;
    }
    while (capacity < numValues) {
        capacity *= VAR_LIST_RESIZE_RATIO;
    }
    dataColumnChunk->resize(capacity);
}

VarListColumnChunk::VarListColumnChunk(LogicalType dataType,
    std::unique_ptr<common::CSVReaderConfig> csvReaderConfig, bool enableCompression)
    : ColumnChunk{std::move(dataType), std::move(csvReaderConfig), enableCompression,
          true /* hasNullChunk */},
      varListDataColumnChunk{
          ColumnChunkFactory::createColumnChunk(*VarListType::getChildType(&this->dataType),
              enableCompression, this->csvReaderConfig.get())} {
    assert(this->dataType.getPhysicalType() == PhysicalTypeID::VAR_LIST);
}

void VarListColumnChunk::append(ColumnChunk* other, offset_t startPosInOtherChunk,
    offset_t startPosInChunk, uint32_t numValuesToAppend) {
    nullChunk->append(
        other->getNullChunk(), startPosInOtherChunk, startPosInChunk, numValuesToAppend);
    auto otherListChunk = reinterpret_cast<VarListColumnChunk*>(other);
    auto offsetInDataChunkToAppend = varListDataColumnChunk.getNumValues();
    for (auto i = 0u; i < numValuesToAppend; i++) {
        offsetInDataChunkToAppend += otherListChunk->getListLen(startPosInOtherChunk + i);
        setValue(offsetInDataChunkToAppend, startPosInChunk + i);
    }
    auto startOffset = otherListChunk->getListOffset(startPosInOtherChunk);
    auto endOffset = otherListChunk->getListOffset(startPosInOtherChunk + numValuesToAppend);
    varListDataColumnChunk.resizeBuffer(offsetInDataChunkToAppend);
    varListDataColumnChunk.dataColumnChunk->append(
        otherListChunk->varListDataColumnChunk.dataColumnChunk.get(), startOffset,
        varListDataColumnChunk.getNumValues(), endOffset - startOffset);
    numValues += numValuesToAppend;
}

void VarListColumnChunk::write(const Value& listVal, uint64_t posToWrite) {
    assert(listVal.getDataType()->getPhysicalType() == PhysicalTypeID::VAR_LIST);
    auto numValuesInList = NestedVal::getChildrenSize(&listVal);
    varListDataColumnChunk.resizeBuffer(varListDataColumnChunk.getNumValues() + numValuesInList);
    for (auto i = 0u; i < numValuesInList; i++) {
        varListDataColumnChunk.dataColumnChunk->write(
            *NestedVal::getChildVal(&listVal, i), varListDataColumnChunk.getNumValues());
        varListDataColumnChunk.increaseNumValues(1);
    }
    setValue(varListDataColumnChunk.getNumValues(), posToWrite);
}

void VarListColumnChunk::setValueFromString(const char* value, uint64_t length, uint64_t pos) {
    auto listVal =
        TableCopyUtils::getVarListValue(value, 1, length - 2, dataType, *csvReaderConfig);
    write(*listVal, pos);
}

void VarListColumnChunk::resetToEmpty() {
    ColumnChunk::resetToEmpty();
    varListDataColumnChunk.reset();
}

void VarListColumnChunk::append(common::ValueVector* vector, common::offset_t startPosInChunk) {
    auto nextListOffsetInChunk = getListOffset(startPosInChunk);
    auto offsetBufferToWrite = (common::offset_t*)(buffer.get());
    for (auto i = 0u; i < vector->state->selVector->selectedSize; i++) {
        auto pos = vector->state->selVector->selectedPositions[i];
        uint64_t listLen = vector->isNull(pos) ? 0 : vector->getValue<list_entry_t>(pos).size;
        nullChunk->setNull(startPosInChunk + i, vector->isNull(pos));
        nextListOffsetInChunk += listLen;
        offsetBufferToWrite[startPosInChunk + i] = nextListOffsetInChunk;
    }
    varListDataColumnChunk.resizeBuffer(nextListOffsetInChunk);
    auto dataVector = ListVector::getDataVector(vector);
    dataVector->setState(std::make_unique<DataChunkState>());
    dataVector->state->selVector->resetSelectorToValuePosBuffer();
    for (auto i = 0u; i < vector->state->selVector->selectedSize; i++) {
        auto pos = vector->state->selVector->selectedPositions[i];
        if (vector->isNull(pos)) {
            continue;
        }
        auto listEntry = vector->getValue<list_entry_t>(pos);
        dataVector->state->selVector->selectedSize = listEntry.size;
        for (auto j = 0u; j < listEntry.size; j++) {
            dataVector->state->selVector->selectedPositions[j] = listEntry.offset + j;
        }
        varListDataColumnChunk.append(dataVector);
    }
    numValues += vector->state->selVector->selectedSize;
}

} // namespace storage
} // namespace kuzu

==> kuzu-0.0.11.tar.gz: sdist/kuzu-source/src/storage/store/var_list_column_chunk.cpp <==
#include "storage/store/var_list_column_chunk.h"

#include "arrow/array.h"
#include "common/types/value/nested.h"
#include "common/types/value/value.h"
#include "storage/store/table_copy_utils.h"

using namespace kuzu::common;

namespace kuzu {
namespace storage {

void VarListDataColumnChunk::reset() const {
    dataColumnChunk->resetToEmpty();
}

void VarListDataColumnChunk::resizeBuffer(uint64_t numValues) {
    if (numValues <= capacity) {
        return;
    }
    while (capacity < numValues) {
        capacity *= VAR_LIST_RESIZE_RATIO;
    }
    dataColumnChunk->resize(capacity);
}

VarListColumnChunk::VarListColumnChunk(LogicalType dataType, bool enableCompression)
    : ColumnChunk{std::move(dataType), enableCompression, true /* hasNullChunk */},
      varListDataColumnChunk{ColumnChunkFactory::createColumnChunk(
          *VarListType::getChildType(&this->dataType), enableCompression)} {
    assert(this->dataType.getPhysicalType() == PhysicalTypeID::VAR_LIST);
}

void VarListColumnChunk::append(ColumnChunk* other, offset_t startPosInOtherChunk,
    offset_t startPosInChunk, uint32_t numValuesToAppend) {
    nullChunk->append(
        other->getNullChunk(), startPosInOtherChunk, startPosInChunk, numValuesToAppend);
    auto otherListChunk = reinterpret_cast<VarListColumnChunk*>(other);
    auto offsetInDataChunkToAppend = varListDataColumnChunk.getNumValues();
    for (auto i = 0u; i < numValuesToAppend; i++) {
        offsetInDataChunkToAppend += otherListChunk->getListLen(startPosInOtherChunk + i);
        setValue(offsetInDataChunkToAppend, startPosInChunk + i);
    }
    auto startOffset = otherListChunk->getListOffset(startPosInOtherChunk);
    auto endOffset = otherListChunk->getListOffset(startPosInOtherChunk + numValuesToAppend);
    varListDataColumnChunk.resizeBuffer(offsetInDataChunkToAppend);
    varListDataColumnChunk.dataColumnChunk->append(
        otherListChunk->varListDataColumnChunk.dataColumnChunk.get(), startOffset,
        varListDataColumnChunk.getNumValues(), endOffset - startOffset);
    numValues += numValuesToAppend;
}

void VarListColumnChunk::write(const Value& listVal, uint64_t posToWrite) {
    assert(listVal.getDataType()->getPhysicalType() == PhysicalTypeID::VAR_LIST);
    auto numValuesInList = NestedVal::getChildrenSize(&listVal);
    varListDataColumnChunk.resizeBuffer(varListDataColumnChunk.getNumValues() + numValuesInList);
    for (auto i = 0u; i < numValuesInList; i++) {
        varListDataColumnChunk.dataColumnChunk->write(
            *NestedVal::getChildVal(&listVal, i), varListDataColumnChunk.getNumValues());
        varListDataColumnChunk.increaseNumValues(1);
    }
    setValue(varListDataColumnChunk.getNumValues(), posToWrite);
}

void VarListColumnChunk::resetToEmpty() {
    ColumnChunk::resetToEmpty();
    varListDataColumnChunk.reset();
}

void VarListColumnChunk::append(common::ValueVector* vector, common::offset_t startPosInChunk) {
    auto nextListOffsetInChunk = getListOffset(startPosInChunk);
    auto offsetBufferToWrite = (common::offset_t*)(buffer.get());
    for (auto i = 0u; i < vector->state->selVector->selectedSize; i++) {
        auto pos = vector->state->selVector->selectedPositions[i];
        uint64_t listLen = vector->isNull(pos) ? 0 : vector->getValue<list_entry_t>(pos).size;
        nullChunk->setNull(startPosInChunk + i, vector->isNull(pos));
        nextListOffsetInChunk += listLen;
        offsetBufferToWrite[startPosInChunk + i] = nextListOffsetInChunk;
    }
    varListDataColumnChunk.resizeBuffer(nextListOffsetInChunk);
    auto dataVector = ListVector::getDataVector(vector);
    dataVector->setState(std::make_unique<DataChunkState>());
    dataVector->state->selVector->resetSelectorToValuePosBuffer();
    for (auto i = 0u; i < vector->state->selVector->selectedSize; i++) {
        auto pos = vector->state->selVector->selectedPositions[i];
        if (vector->isNull(pos)) {
            continue;
        }
        auto listEntry = vector->getValue<list_entry_t>(pos);
        dataVector->state->selVector->selectedSize = listEntry.size;
        for (auto j = 0u; j < listEntry.size; j++) {
            dataVector->state->selVector->selectedPositions[j] = listEntry.offset + j;
        }
        varListDataColumnChunk.append(dataVector);
    }
    numValues += vector->state->selVector->selectedSize;
}

} // namespace storage
} // namespace kuzu
